#include "messagefacility/MessageLogger/MessageLogger.h"
#include "SuperNovaDDT/online/SNMessageService.h"

namespace novaddt{
  SNMessageService::SNMessageService
  (fhicl::ParameterSet const& p, art::ActivityRegistry& reg):
    fMailbox({"SuperNovaDDT",
	  p.get<unsigned>("dds.partition"),
	  p.get<std::string>("dds.mailbox")}),
    fMessageBuffer( p.get<std::string>("shmem.segment").data(),
		    p.get<std::string>("shmem.object").data())
  {
  }
  
  void SNMessageService::PushPoint(uint64_t time, uint64_t hits){
    //lock buffer before modifying it
    auto lock=fMessageBuffer.ScopedLock();
    //check if we have enough point in buffer for message
    if(fMessageBuffer.isFull()){
      auto message=fMessageBuffer.makeMessage();
      fMessageBuffer.clear();
      fMailbox<<message;
    }
    //append buffer
    try {
      fMessageBuffer.addPoint({time,hits});
    }
    catch (std::exception& e){
      //in case of any exception: print out error and skip this point
      mf::LogError("SNMessageService")<<e.what();
    }
    
  }
  DEFINE_ART_SERVICE_INTERFACE_IMPL(SNMessageService,SNMessageInterface);
}
